#include "lidar_lite.h"

CLidarLite::CLidarLite(const unsigned char &dev_id, const std::string &serial)
{
  this->dev_id_ = dev_id;
  this->serial_ = serial;
  this->status_ = IDLE;
}
 
CLidarLite::~CLidarLite(void)
{
	if (this->status_ == RUNNING)
    close();
}

void CLidarLite::write(unsigned char addr, unsigned char cmd)
{
  this->adapter_->write_reg(this->dev_id_, addr, &cmd, 1);
}

int CLidarLite::read(unsigned char addr, int len)
{
  if (len!=1 && len!=2)
    throw CLidarLiteException(_HERE_, "Expected read data with invalid length. This function only allows to read 1 or 2 bytes");

  if (len == 2)
    addr = addr_to_read_2bytes(addr); // See header file for a description of this functionality.
  
  unsigned char data[len];
  this->adapter_->write(this->dev_id_,&addr,1);
  this->adapter_->read(this->dev_id_,data,len);

  int val;
  if (len==1)
    val = data[0];
  else if (len==2)
  {
    unsigned short int read_val = data[0]*256+data[1];
    val = (int)read_val;
  }

  return val;
}


void CLidarLite::open(void)
{
  if (this->status_ != IDLE)
    throw CLidarLiteException(_HERE_, "Device cannot be OPENED because it is already running.");

  try
  {
  	this->adapter_ = new CUSBI2C("adapter");

		// open serial port
    std::cout << "[CLidarLite] Opening Lidar Lite. " << std::endl; 
    this->adapter_->open(this->serial_);
    usleep(10000); // Avoid initialization errors
    std::cout << "  - Serial number: " << this->serial_ << std::endl;
    std::cout << "  - Firmware revision: " << (int)this->adapter_->get_revision() << std::endl;

		// Set gpio 2 and 3 as i2c    
    this->adapter_->config_gpio(gpio2,i2c);
    this->adapter_->config_gpio(gpio3,i2c);


    // TEST WORK IN PROGRESS **************************

    write(ACQ_COMMAND,0x04);

    //Read STATUS. Repeat until bit 0 (LSB) goes low.
    unsigned char busy;
    do 
      busy = read(STATUS,1);
    while( busy & 0x01 );
    
    // regular read example
    // std::cout << "Register " << std::hex << ACQ_CONFIG_REG << " with value: " << read(ACQ_CONFIG_REG,1) << std::dec << std::endl;

    // read two bytes
    std::cout << "Lidar-Lite measurement: " << read(FULL_DELAY_HIGH,2) << " cm" << std::endl;
    
    // ************************************************

  }catch(CException &e)
  {
  	throw CLidarLiteException(_HERE_, e.what());
  }
  
  this->status_ = RUNNING;
}

void CLidarLite::close(void)
{
  if (this->status_ != RUNNING)
    throw CLidarLiteException(_HERE_, "Device cannot be CLOSED because it is not running.");

	try
	{
    if(this->adapter_ != NULL)
    {
      this->adapter_ = NULL;
      delete this->adapter_;    
    }
  }catch(CException &e)
  {
  	throw CLidarLiteException(_HERE_, e.what());
  }
}

void CLidarLite::reset(void)
{
  if (this->status_ != RUNNING)
    throw CLidarLiteException(_HERE_, "Device cannot be RESET because it is not running.");

  write(ACQ_COMMAND,0x00);
}


void CLidarLite::blink_led(void)
{
  if (this->status_ != RUNNING)
    throw CLidarLiteException(_HERE_, "Usb adapter LED will not BLINK because Lidar-Lite is not running.");

  try
  {
    for (int ii = 0; ii < 2; ++ii)
    {
      this->adapter_->turn_led_off();
      usleep(200000);
      this->adapter_->turn_led_on();
      usleep(200000);
    }
  }catch(CException &e)
  {
    throw CLidarLiteException(_HERE_, e.what());
  }
}

unsigned short int CLidarLite::addr_to_read_2bytes(unsigned char first_byte_addr)
{
  unsigned short int val = (0x01<<7) | first_byte_addr;
  return val;
}