#ifndef _LIDAR_LITE_H
#define _LIDAR_LITE_H

#include <iostream>
#include "usb_i2c.h"
#include "exceptions/lidar_lite_exceptions.h"

enum V3_REGISTER_DEF
{  
	ACQ_COMMAND = 		0x00,	// Device command
	STATUS = 			0x01,	// System status
	SIG_COUNT_VAL = 	0x02,	// Maximum acquisition count
	ACQ_CONFIG_REG = 	0x04,	// Acquisition mode control
	VELOCITY = 			0x09,	// Velocity measurement output
	PEAK_CORR = 		0x0c,	// Peak value in correlation record	
	NOISE_PEAK = 		0x0d,
	SIGNAL_STRENGTH = 	0x0e,
	FULL_DELAY_HIGH = 	0x0f,
	FULL_DELAY_LOW = 	0x10,
	OUTER_LOOP_COUNT = 	0x11,
	REF_COUNT_VAL = 	0x12,
	LAST_DELAY_HIGH = 	0x14,
	LAST_DELAY_LOW = 	0x15,
	UNIT_ID_HIGH = 		0x16,
	UNIT_ID_LOW = 		0x17,
	I2C_ID_HIGH = 		0x18,
	I2C_ID_LOW = 		0x19,
	I2C_SEC_ADDR = 		0x1a,
	THRESHOLD_BYPASS = 	0x1c,
	I2C_CONFIG = 		0x1e,
	COMMAND = 			0x40,
	MEASURE_DELAY = 	0x45,
	PEAK_BCK = 			0x4c,
	CORR_DATA = 		0x52,
	CORR_DATA_SIGN = 	0x53,
	ACQ_SETTINGS =		0x5d,
	POWER_CONTROL =		0x65, 
};

enum DEV_STATUS 
{  
	IDLE,
	RUNNING
};

class CLidarLite
{
	private:
		std::string serial_;	// Serial port (e.g., run dmesg).
		unsigned char dev_id_;	// Device ID (from datasheet).
		CUSBI2C* adapter_;		// Device object.
		int status_;			// Device status.

		void blink_led(void);


		void write(unsigned char addr, unsigned char cmd);
		int read(unsigned char addr, int len);

	    // Successive 2-byte readings (Autoincrement of address: A note about 0x8f vs 0x0f)
	    // Set the highest bit of any register to "1" if you set the high byte of a register 
	    // and then take succesive readings from that register, then LIDAR-Lite automatically 
	    // increments the register one for each read. 
	    // An example: If we want to read the high and low bytes for the distance, we could 
	    // take two single readings from 0x0f and 0x10, or we could take 2 byte read from 
	    // register 0x8f. 0x8f = 10001111 and 0x0f = 00001111, meaning that 0x8f is 0x0f with 
	    // the high byte set to "1", ergo it autoincrements.
		unsigned short int addr_to_read_2bytes(unsigned char first_byte_addr);

  public:
    CLidarLite(const unsigned char &dev_id, const std::string &serial);
    ~CLidarLite(void);

		void open(void);
		void close(void);
		void reset(void);
};

#endif
